/* globals PDFJS */
import Ember from 'ember';
import layout from '../templates/components/pdf-simple';

PDFJS.workerSrc = '/assets/javascript/pdf.worker.min.js';

export default Ember.Component.extend({
  layout,

  src: '',
  page: 1,
  scale: 1,
  rotation: 0,
  containerClasses: '',
  numPages: null, // mutable object

  pdf: null,
  pdfPageView: null,

  didInsertElement: function() {
    this.loadPDF().then(() => {
      this.addObserver('src', this, this.loadPDF);
      this.addObserver('page', this, this.drawPDF);
      this.addObserver('scale', this, this.updatePdfPageView);
      this.addObserver('rotation', this, this.updatePdfPageView);
    });
  },

  updatePdfPageView: function() {
    const pdfPageView = this.get('pdfPageView');
    const scale = this.get('scale');
    const rotation = this.get('rotation');

    if (!pdfPageView) {
      return undefined;
    }

    if (typeof rotation !== 'number' || rotation % 90) {
      Ember.Logger.warn('pdf-simple: the provided `rotation` property should be a number and have a remainder of 0 when divided by 90!\nContinuing with rotation = 0');
    }

    return pdfPageView.update(scale, rotation);
  },

  loadPDF: function() {
    const src = this.get('src');

    if (!src) {
      Ember.Logger.error('You have to pass a PDF url as the `src` property to pdf-simple!');
      return Ember.RSVP.reject();
    }

    return PDFJS.getDocument(src).then((pdf) => {
      this.set('pdf', pdf);
      this.set('numPages', pdf.numPages);
      this.drawPDF();
    });
  },

  drawPDF: function() {
    const {
      pdf,
      page,
      scale,
    } = this.getProperties(
      'pdf',
      'page',
      'scale',
    );

    const container = this.$('.pdf-simple-container')[0];

    if (container.childElementCount) {
      container.removeChild(container.childNodes[0]);
    }

    pdf.getPage(page).then((pdfPage) => {
      const pdfPageView = new PDFJS.PDFPageView({
        container,
        id: page,
        scale,
        defaultViewport: pdfPage.getViewport(scale),
        textLayerFactory: new PDFJS.DefaultTextLayerFactory(),
        annotationLayerFactory: new PDFJS.DefaultAnnotationLayerFactory(),
      });

      pdfPageView.setPdfPage(pdfPage);
      this.set('pdfPageView', pdfPageView);

      // updating page view to reflect rotation other than 0
      this.updatePdfPageView();

      pdfPageView.draw().then(() => {
        // Here we make sure the page div which contains the canvasWrapper and
        // textLayer divs is an offsetParent() to its children
        this.$('.page').css('position', 'relative');
      });
    });
  },

});
